/*
 * spi.c
 *
 *  Created on: Jul 2, 2022
 *      Author: hussamaldean
 */
#include "spi.h"
#include "stm32l0xx.h"
void spi_init()
{
	/*Enable clock access to GPIOA*/
	RCC->IOPENR |= RCC_IOPENR_IOPAEN;

	/*Set PA0 as output */
	GPIOA->MODER|=(1<<0);
	GPIOA->MODER&=~(1<<1);


	/*Set PA5,PA6,PA7 mode to alternate function*/
	/*PA5*/
	GPIOA->MODER &=~(1U<<10);
	GPIOA->MODER |=(1U<<11);

	/*PA6*/
	GPIOA->MODER &=~(1U<<12);
	GPIOA->MODER |=(1U<<13);

	/*PA7*/
	GPIOA->MODER &=~(1U<<14);
	GPIOA->MODER |=(1U<<15);

	/*Set PA5,PA6,PA7 alternate function type to SPI1 (AF0)*/

	/*PA5*/
	GPIOA->AFR[0] &= ~(1U<<20);
	GPIOA->AFR[0] &= ~(1U<<21);
	GPIOA->AFR[0] &= ~(1U<<22);
	GPIOA->AFR[0] &= ~(1U<<23);

	/*PA6*/
	GPIOA->AFR[0] &= ~(1U<<24);
	GPIOA->AFR[0] &= ~(1U<<25);
	GPIOA->AFR[0] &= ~(1U<<26);
	GPIOA->AFR[0] &= ~(1U<<27);

	/*PA7*/
	GPIOA->AFR[0] &= ~(1U<<28);
	GPIOA->AFR[0] &= ~(1U<<29);
	GPIOA->AFR[0] &= ~(1U<<30);
	GPIOA->AFR[0] &= ~(1U<<31);

	/*Enable clock access to SPI1 module*/
	RCC->APB2ENR |= RCC_APB2ENR_SPI1EN;



	/*Set mode to MASTER*/
	SPI1->CR1 |= (1U<<2);


	/*Select software slave management by
	* setting SSM=1 and SSI=1*/
	SPI1->CR1 |= (1<<8);
	SPI1->CR1 |= (1<<9);

	/*Enable SPI module*/
	SPI1->CR1 |= (1<<6);

}


void spi_transmit(uint8_t *data,uint32_t size)
{
	uint32_t i=0;


	while(i<size)
	{
		 /*Wait for the TXE bit to set in the Status Register*/
		/*This will indicate that the transmit buffer is empty*/
		 while (!((SPI1->SR)&(1<<1))) {};
		   /*Write the data to the Data Register*/
		   SPI1->DR = data[i];
		   i++;
	}
	 /*Wait for the TXE bit to set in the Status Register*/
	 while (!((SPI1->SR)&(1<<1))) {};

	 /*Wait for the BSY bit to reset in the Status Register*/
	 while (((SPI1->SR)&(1<<7))) {};

	 /*Clear OVR flag*/
	(void) SPI1->DR;
	(void) SPI1->SR;
}

void spi_receive(uint8_t *data, uint32_t size)
{
	while(size)
	{
		SPI1->DR = 0;  // send dummy data
		while (!((SPI1->SR) &(SPI_SR_RXNE))){};
		 /*Read data from Data Register*/
		 *data++ = (SPI1->DR);
		size--;
	}
}

void cs_low()
	{
	GPIOA->BSRR=GPIO_BSRR_BR_0;
	}

void cs_high()
	{
	GPIOA->BSRR=GPIO_BSRR_BS_0;
	}
