#include "uart.h"
#include  "stm32l0xx.h"
#include "stdio.h"

/*STM32L053 frequency*/
#define SYS_FREQ		2097000
/*APB1 Clock bus speed*/
#define APB1_CLK		SYS_FREQ
/*Desired baud rate*/
#define UART_BAUDRATE	115200

/**
 * @brief   Compute the baudrate of UART
 * @param   peripheral clock and desired baudrate
 * @retval  value for BRR register
 */

static uint16_t compute_uart_bd(uint32_t PeriphClk, uint32_t BaudRate)
{
	return ((PeriphClk  +  (BaudRate/2U))/BaudRate);
}


/**
 * @brief   Initialize USART2
 * @note    PA2 and PA3
 * @param   None
 * @retval  None
 */
void uart_init()
	{
	/*Enable clock access to GPIOC*/
	RCC->IOPENR |= RCC_IOPENR_GPIOAEN;
	GPIOA->MODER|=GPIO_MODER_MODE2_1;
	GPIOA->MODER&=~GPIO_MODER_MODE2_0;
	#define AF04 0x04
	GPIOA->AFR[0]|=(AF04<<8);

	RCC->APB1ENR |= RCC_APB1ENR_USART2EN;
	USART2->BRR=compute_uart_bd(APB1_CLK,UART_BAUDRATE);

	/*Configure transfer direction*/
    USART2->CR1 = USART_CR1_TE;

	/*Enable uart module*/
    USART2->CR1 |= USART_CR1_UE;

	}


/**
 * @brief   Write single character to the UART hardware
 * @param   desired character
 * @retval  None
 */

void uart2_write(int ch)
{
	/*Make sure transmit data register is empty*/
	while(!(USART2->ISR & USART_ISR_TXE)){}

	/*Write to the transmit data register*/
	USART2->TDR =  (ch & 0xFF);
}


/**
 * @brief   Retargeted printf over uart
 * @param   desired character
 * @retval  passed character
 */


int __io_putchar(int ch)
{
	uart2_write(ch);
	return ch;
}

